#include <Bounce.h>
#include <Shifter.h>

//-------------------------------------------------------//
//Define PINS numbers//
//-------------------------------------------------------//

//TACT SWITCHES CONFIG 
byte PBPIN[10] ={
  0,1,5,6,7,8,9,10,11,12
};


int MUX_DIG = 4; //read pin

//LEDs shifts setup
#define SHIFT_OUT_DATA 15 //SER_IN
#define SHIFT_OUT_LATCH 14 //L_CLOCK
#define SHIFT_OUT_CLOCK 13 //CLOCK
#define SHIFT_OUT_REGISTERS 4
Shifter shifter(SHIFT_OUT_DATA, SHIFT_OUT_LATCH, SHIFT_OUT_CLOCK, SHIFT_OUT_REGISTERS); 

//An index to match led numbers on PCB with led on 595 chain
byte LEDPINS[33]={
  //Toggle LEDs
  16,//0 LED0
  17,//1 LED1
  18,//2 LED2
  29,//3 LED27 
  30,//4 LED28
  31,//5 LED29
  15,//6 LED30
  14,//7 LED31

  //2-WAY LEDs 
  2,//8 LED17
  1,//9 LED18

  //3-WAY LEDS
  10,//10 LED11 
  9,//11 LED12
  8,//12 LED13
  7,//13 LED14
  6,//14 LED15
  5,//15 LED16
  0,//16 LED19
  4,//17 LED20
  3,//18 LED21
  24,//19 LED22
  25,//20 LED23
  26,//21 LED24

  100,//22 NO LED
  28,//23 LED26
  27,//24 LED25

  //4-WAY LEDS
  19,//25 LED3
  20,//26 LED4
  21,//27 LED5
  22,//28 LED6
  23,//29 LED7
  13,//30 LED8
  12,//31 LED9
  11//32 LED10
};

//Analog MUXES
//Configure MUX adress pins 
byte SPINS[3] = {
  20, 21, 22}; 

//Configure MUX OUT PIN
byte VOUT_ANA1 = 19; 
byte VOUT_ANA2 = 23; 
byte VOUT_ANA3 = 18;

//Analog Teensy
byte TEENSY_ANA[2] = {
  16,
  17};

//-------------------------------------------------------//
//DEBOUNCE TACT SWITCHES
//-------------------------------------------------------//
Bounce PBOUNCED[10] =
{
  Bounce((PBPIN[0]), 50),
  Bounce((PBPIN[1]), 50), 
  Bounce((PBPIN[2]), 50), 
  Bounce((PBPIN[3]), 50), 
  Bounce((PBPIN[4]), 50),
  Bounce((PBPIN[5]), 50), 
  Bounce((PBPIN[6]), 50), 
  Bounce((PBPIN[7]), 50),
  Bounce((PBPIN[8]), 50), 
  Bounce((PBPIN[9]), 50),
};

//-------------------------------------------------------//
//VARIABLES
//-------------------------------------------------------//  

byte PBSTATUS[18]={ //INIT TO 1 TO MAKE SURE THE LEDs LIGHTS UP TO DEFAULT VALUE AT REBOOT
  //Toggle PBs
  1,//0 PB2
  1,//1 PB1
  1,//2 PB0
  1,//3 PB12
  1,//4 PB11
  1,//5 PB14
  1,//6 PB15
  1,//7 PB13

  //2-WAY PBs 
  1,//8 PB7

  //3-WAY PBs
  1,//9 PB5
  1,//10 PB6
  1,//11 PB8
  1,//12 PB9
  1,//13 PB10

  //4-WAY PBs
  1,//14 PB3
  1,//15 PB4

  0, //record sequence
  0  //HOLD

};

//ARRAY FOR PUSHBUTTONS STATES
byte buttonState_PB[19]  = {
  0, //0-trig
  0, //1-inv
  1, //2-sync
  0, //3-shift
  0, //4-poly
  0, //5-arp
  0, //6-seq
  1, //7-sync
  0, //8-vca  
  1, //9-pwm
  2, //10-sub
  0, //11-env
  1, //12-trans
  0, //13-port
  0, //14-lfo 4way
  1, //15-vco 4way
  0, //16-recordsequence
  0, // hold
  0 // lock
};

byte PreviousbuttonState_PB[19]  = {
  0,//0
  0,//1
  1,//2
  0,//3
  0,//4
  0,//5
  0,//6
  1,//7
  0,//8
  1,//9
  2,//10
  0,//11
  1,//12
  0,//13
  0,//14
  1,
  0,
  0,
  0
};

byte LastPreviousbuttonState_PB[19]  = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};

byte DigIndex[10]={
  8,11,10,9,15,4,3,5,6,7  
};

//Debug Psh buttons 
int DebugPB[16] = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};


//Filtering and sampling analog inputs
byte AnalogThreshold = 4; //threshold
elapsedMillis timer;
elapsedMillis timer2;
byte Smoothing = 50;
byte AnalogSamplingPeriod = 15;

elapsedMillis ledtimer;
byte blinking = 200;
boolean flip = 0;

//MUX DIGITAL INPUT ARRAYS

boolean digmux[8] = {
  0,0,0,0,0,0,0,0};

boolean PREVIOUS_digmux[8] = {
  0,0,0,0,0,0,0,0};

int MuxDebounce[8] = {
  0,0,0,0,0,0,0,0};
  
byte MuxIndex[8]={
  12,0,13,0,14,2,0,1 
};

byte ToReadMux[8]={
  1,0,1,0,1,1,1,1 
};


byte BounceTime = 125;

//ANALOG MUXES ARRAYS

//Debug mux pins 
byte AnaIndex[26] = {
  5,//Fader 5
  3,//Fader 3
  2,//Fader 2
  1,//.....
  4,
  0,
  6,
  7,

  13,
  11,
  9,
  15,
  8,
  14,
  10,
  12,

  17,
  18,
  19,
  16,
  23,//Pot2
  22,//Pot1 
  20,//Fader20
  21,//Pot0

  24,//Pot3
  25 //Pot4
};

int Analog[26] = {
  0,//Fader0
  0,//Fader1
  0,//Fader2
  0,//Fader3
  0,//Fader4

  0,//Fader5
  0,//Fader6
  0,//Fader7
  0,//Fader8
  0,//Fader9

  0,//Fader10
  0,//Fader11
  0,//Fader12
  0,//Fader13
  0,//Fader14

  0,//Fader15
  0,//Fader16
  0,//Fader17
  0,//Fader18
  0,//Fader19

  0,//Fader20
  0,//Pot0
  0,//Pot1
  0,//Pot2
  0,//Pot3

  0 //Pot4
};

byte Analog_Debug[26] = {
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0};

int Previous_Analog[26] = {
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0};

byte Analog_Mapped[26] = {
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0};

byte Previous_Analog_Mapped[26] = {
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0,0,0,0,0,
  0,0,0,0,0,0};

//MUX control array 
int muxChannel[8][3]={ 
  {
    0,0,0        }
  ,
  {
    1,0,0        }
  ,
  {
    0,1,0        }
  ,
  {
    1,1,0        }
  ,
  {
    0,0,1        }
  ,
  {
    1,0,1        }
  , 
  {
    0,1,1        }
  , 
  {
    1,1,1        }
  ,
};

//boolean REBOOTING = 0;

boolean InitFlag = 0;
byte PressTime = 0;
byte Blinky = 250;
boolean LedState = 1;


// SETUP (pin config) ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //
void setup(){ 

  Serial.begin(9600); 

  //PB pin config
  for(int i=0;i<10;i++){
    pinMode(PBPIN[i],INPUT_PULLUP);
  }

  //VOUT MUX PIN CONFIG
  pinMode(VOUT_ANA1,INPUT);
  pinMode(VOUT_ANA2,INPUT);
  pinMode(VOUT_ANA3,INPUT);

  //ANA PIN CONFIG
  pinMode(TEENSY_ANA[0],INPUT);
  pinMode(TEENSY_ANA[1],INPUT);  

  //MUX adress PINS CONFIG
  for (int i =0; i<3;i++){
    pinMode(SPINS[i],OUTPUT);
  }

  //MUX DIG PINS CONFIG
  pinMode(MUX_DIG, INPUT_PULLUP);

  //RESET PINS CONFIG
  for (int i =0; i<3;i++){
    digitalWrite(SPINS[i],LOW);
  }

  shifter.clear();

  Boot();

}

// LOOPS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ //
void loop(){

  //-------------------------------------------------------//
  //READING DIGITAL INPUTS//
  //-------------------------------------------------------//
  ReadDigital();

  //-------------------------------------------------------//
  //RECALL INIT//
  //-------------------------------------------------------//
  Recall();

  //-------------------------------------------------------//
  //TOGGLE SWITCHES AND DIGITAL MIDI
  //-------------------------------------------------------//
  MidiDigital();

  //-------------------------------------------------------//
  //LEDs CONTROL//
  //-------------------------------------------------------//
  Leds();

  //-------------------------------------------------------//
  //READING AND SMOOTHING ANALOG//
  //-------------------------------------------------------//
  ReadAnalog();

  //-------------------------------------------------------//
  //SAMPLING, TESTING THRESHOLD AND SENDS MIDI//
  //-------------------------------------------------------//
  MidiAnalog();

  InitFlag=0;

}




